module TalltyImportExport
  class Export
    attr_reader :klass, :headers

    def initialize klass
      @klass = klass
    end

    # export_headers / headers
    # key: 属性的英文名
    # name: 属性的中文名
    # attr_type: 属性的类型
    # format: excel是否需要特定的格式，目前主要是类似于身份证号，可以用string
    # method: 导出时本地调用的方法
    # chain: 导出时对象属性通过链式调用

    def export_records records, options={}
      exportable_defaults(options)

      options = options.with_indifferent_access

      Axlsx::Package.new do |pack|
        workbook = pack.workbook
        # excel导出样式
        alignment = { vertical: :center, horizontal: :center }
        border    = { color: '969696', style: :thin }
        title1 = workbook.styles.add_style(alignment: alignment, border: border, sz: 16, b: true)
        title2 = workbook.styles.add_style(alignment: alignment, border: border, bg_color: "2a5caa",  sz: 16, fg_color: "fffffb")
        title3 = workbook.styles.add_style(alignment: alignment.merge(wrap_text: true), border: border, sz: 14)

        workbook.add_worksheet do |sheet|
          if respond_to?(:first_header)
            row_index = Axlsx.col_ref(headers.size - 1)
            sheet.merge_cells("A1:#{row_index}1")
            sheet.add_row [first_header], style: title1, height: 40
          end

          sheet.add_row @headers.map{|header| header[:name]}, style: title2, height: 39

          records.find_each do |record|
            row = []
            headers.each{ |header| row.push(handle_data(record, header)) }
            sheet.add_row row, style: title3, height: @row_height, types: @headers.map{|header| header[:format]}
          end
          sheet.column_widths *@headers.map{|header| header[:width] || @width}
        end
        file_path = File.join(Rails.root, 'public', 'export')
        FileUtils.mkdir_p(file_path) unless Dir.exist?(file_path)
        file_name = "#{Time.now.strftime('%Y%m%d%H%M%S')}#{@filename}.xlsx"
        pack.serialize(File.join(file_path, file_name))
        url = ActionController::Base.helpers.asset_url("/export/#{file_name}")
        return url
      end
    end

    def exportable_defaults options = {}
      options = options.with_indifferent_access

      @row_height ||= options.delete(:row_height) || 35
      @width ||= options.delete(:width) || 30
      @filename ||= options.delete(:filename)
      self.headers = export_headers
    end

    def headers= val
      @headers = val.map { |header| header.with_indifferent_access }
    end

    def export_headers
      klass.try(:headers) || klass.try(:model_headers)
    end

    # 处理一个记录的数据
    def handle_data record, header
      data =
        if header[:method].present?
          send(header[:method], record, header[:key])
        elsif header[:chain].present?
          header[:chain].reduce(record) do |obj, method|
            obj.send(method)
          end
        elsif header[:json]
          record.send(header[:json])[header[:key]]
        else
          record.send(header[:key])
        end
        handle_format(data, header)
    rescue
      ''
    end

    # 根据数据类型 attr_type 进行数据的格式化
    def handle_format data, header
      case header[:attr_type].to_s
      when 'datetime'
        data ? data.strftime('%F %H:%M') : nil
      when 'date'
        data ? data.strftime('%F') : nil
      else
        data
      end
    end
  end
end
