module TalltyImportExport
  class Import
    require 'roo'
    attr_reader :klass, :context, :headers, :primary_keys

    def initialize klass
      @klass = klass
      @context = Context.new({})
    end

    # key: 属性的英文名
    # name: 属性的中文名
    # attr_type: 属性的类型
    # format: excel是否需要特定的格式，目前主要是类似于身份证号，可以用string
    # convert: 导入时候，把excel的内容转换成导入时候代码逻辑需要的内容
    # primary_key: 是否是主键

    def import_xlsx xlsx_file, associations, options={}
      # 先处理获取出来Excel每行的数据, line_info
      options = options.with_indifferent_access

      self.headers = options.delete(:headers) || import_headers
      @primary_keys = options.delete(:primary_keys) || headers.map { |header| header[:primary_key] ? header[:key].to_sym : nil }.compact

      excel_hash = headers.reduce({}) do |h, header|
        h[header[:key]] = header[:name]
        h
      end
      file_path = xlsx_file.is_a?(String) ? xlsx_file : xlsx_file.path
      xlsx = ::Roo::Excelx.new(file_path)
      xlsx.each_with_pagename do |_sheetname, sheet|
        sheet.each(**excel_hash).with_index do |line_info, index|
          next if index == 0

          # 转换处理导入的数据格式
          line_info = convert_data(line_info)

          # 处理每行对于导入的动作,处理line_info
          import_record(line_info, associations)
          context.last_line_info = line_info
        end
      end
    end

    def convert_data line_info
      line_info.with_indifferent_access.reduce({}) do |h, (k, v)|
        header = headers.find do |_header|
          _header[:key].to_sym == k.to_sym
        end
        # header[:convert] = handle_xxx
        # handle_xxx(val, processing_line_info, raw_line_info)
        val = header[:convert] ? send(header[:convert], v, h, line_info) : v
        if header[:json]
          h[header[:json]] ||= {}
          h[header[:json]][k] = val
        else
          h[k.to_sym] = val
        end
        h
      end.with_indifferent_access
    end

    def import_headers
      klass.try(:headers) || klass.try(:model_headers)
    end

    def headers= val
      @headers = val.map { |header| header.with_indifferent_access }
    end

    def skip val, processing_line_info, raw_line_info
      # do nothing there, use for header[:convert]
    end

    ###  这个方法是可以由复杂业务进行重载的 ###
    def import_record line_info, associations
      if primary_keys.present?
        _record = associations.find_or_initialize_by(line_info.clone.extract!(*primary_keys))
        _record.update!(line_info.clone.except!(*primary_keys))
      else
        associations.create!(line_info)
      end

    end
  end
end
